/*
// @file		Pop3WindowController.mm
//
// @version		Mailit for C/C++ 1.0 sample file
//
// @author		(C) 2007 Dacons LLP.
//
// @bug			If you have problems or suggestions, please contact
//				the Online Support at: http://www.dacons.net/support
*/

#import "Pop3WindowController.h"

#include PCH_FILE_NAME


#include "POP3Callback.h"
#import "stringext.h"
#include <assert.h>
#include "ErrorsDescriptions.hpp"

@implementation Pop3WindowController
- (id)init
{
	self = [super initWithWindowNibName:@"Window"];
	if (self == NULL)
		return self;
	mMessages = new std::vector<Emai::Message>;
	mAttachments = new std::vector<ItemData>;
	[self window];
	
	return self;
}

- (void)dealloc
{
	delete mMessages;
	delete mAttachments;
	[super dealloc];
}

- (void)windowDidLoad
{
	[mMailServer setStringValue:@"your.mailserver.net"];
	[mAccount setStringValue:@"your.account"];
	[mPassword setStringValue:@"password"];
	[mPort setIntValue:EmaiDefaultPop3Port];
}

-(void) showAlertForException:(Emai::Exception&) inException
{	
	NSAlert *alert = [NSAlert alertWithMessageText:@"An error occured while communicating with pop3 server"
									 defaultButton:@"OK" alternateButton:NULL otherButton:NULL
						 informativeTextWithFormat:@"%@", errorsDescriptions[::abs(inException.GetErrorCode())]];
	[alert setAlertStyle:NSCriticalAlertStyle];
	[alert runModal];
}

-(void) showAlertWithMessageText:(NSString*) inInfText description:(NSString*) inMsgText
{
	NSAlert *alert = [NSAlert alertWithMessageText:inInfText
									 defaultButton:@"OK" alternateButton:NULL otherButton:NULL
						 informativeTextWithFormat:@"%@", inMsgText];
	[alert setAlertStyle:NSWarningAlertStyle];
	[alert runModal];
}

- (void)getAttachment:(NSString*)inDefaultFileName path:(Emai::UniString&)outPath fileName:(Emai::UniString&) outFileName
{	
	NSSavePanel* savePanel = [NSSavePanel savePanel];
	[savePanel setTitle:@"Save attachment"];
	[savePanel setMessage:@"Choose place to save attachment."];
	[savePanel setDelegate:self];
	int result = [savePanel runModalForDirectory:NSHomeDirectory() file:inDefaultFileName];
	NSString* path = NULL;
	NSString* fileName = NULL;
    if (result == NSOKButton) 
	{
		path = [savePanel directory];
		fileName = [[savePanel filename] lastPathComponent];
	}
	
	outPath = [path uniString];
	outFileName = [fileName uniString];
}

- (void)tableViewSelectionDidChange:(NSNotification *)notification;
{
	[self messsageSelect:mMessageList];
}

- (void)fillTextField:(NSTextField*)ioField withFieldType:(EmaiOptions)inType 
	  usingEnumerator:(MessagePartEnumerator&)inEnumerator
{
	
	Emai::UniString name;
	Emai::UniString address;
				
	Emai::ContactList contacts = inEnumerator.GetContacts(inType);
	EmaiUint32 count = contacts.GetCount();
	NSMutableString* str = [NSMutableString string];
	for (EmaiUint32 i = 0; i < count; ++i)
	{
		name = contacts.GetName(i);
		address = contacts.GetAddress(i);
		if ((name.length() > 0) && (address.length() > 0))
			[str appendString:[NSString stringWithFormat:@"%S - %S; ",
				name.c_str(), address.c_str()]];
		else if ((name.length() > 0) && (address.length() == 0))
			[str appendString:[NSString stringWithFormat:@"%S; ",
				name.c_str()]];
		else if ((name.length() == 0) && (address.length() > 0))
			[str appendString:[NSString stringWithFormat:@"%S; ",
				address.c_str()]];
	}
	[ioField setStringValue:str];
}

- (void)fillMessageTextView:(MessagePartEnumerator&)inEnumerator
{
	NSString* msg = NULL;
	Emai::UniString text;
	inEnumerator.GetUnicodeText(text);
	Emai::String html;
	inEnumerator.GetHtml(html);
	if (text.empty())
	{
		[mTextType selectCellWithTag:1];
		[mTextType setEnabled:FALSE];
		msg = [[NSString alloc] initWithBytes:html.c_str() length: html.length() encoding:NSISOLatin1StringEncoding];
	}
	else if (html.empty())
	{
		[mTextType selectCellWithTag:0];
		[mTextType setEnabled:FALSE];
		msg = [NSString stringWithFormat:@"%S", text.c_str()];
	}
	else
	{
		[mTextType setEnabled:TRUE];
		if([[mTextType selectedCell] tag] == 0)
		{
			// Fill text field with plain text
			msg = [NSString stringWithFormat:@"%S", text.c_str()];
		}
		else
		{
			// Fill text field with html text
			msg = [[NSString alloc] initWithBytes:html.c_str() length: html.length() encoding:NSISOLatin1StringEncoding];
		}
	}
	[mMessage setString:msg];
}

- (IBAction)messsageSelect:(id)sender
{
	if (mMessages->size() > 0)
	{
		try
		{
			Emai::Message message = mMessages->at([sender selectedRow]);
			MessagePartEnumerator messagePartEnumerator(message);
			messagePartEnumerator.Enumerate();

			// Fill fields with data
			[self fillTextField:mFrom withFieldType:EmaiFrom 
				usingEnumerator:messagePartEnumerator];
			[self fillTextField:mTo withFieldType:EmaiTo 
				usingEnumerator:messagePartEnumerator];
			[self fillTextField:mCC withFieldType:EmaiCC 
				usingEnumerator:messagePartEnumerator];

			// Fill attachments list with attachments
			mAttachments->clear();
			EmaiUint32 attachmentsCount = messagePartEnumerator.GetAttachmentsCount();
			EmaiUint32 length = 0;
			NSMutableString* attachmentInfo = [NSMutableString string];
			ItemData data;
			if (attachmentsCount > 0)
			{
				Emai::UniString fileName;
				for (EmaiUint32 i = 0; i < attachmentsCount; i++)
				{
					messagePartEnumerator.GetAttachmentAtIndex(i, fileName, NULL, length);
					attachmentInfo = [NSString stringWithFormat:
								@"%S (size = %d)", 
								fileName.c_str(), length];
					data.mFormatedString = [attachmentInfo uniString];
					data.mFileName = fileName;
					data.mIsInline = false;
					data.mAttachmentIndex = i;
					mAttachments->push_back(data);
				}
			}

			// Fill attachments list with inline attachments
			attachmentsCount = messagePartEnumerator.GetInlinesCount();
			length = 0;
			Emai::UniString contentId;
			if (attachmentsCount > 0)
			{
				Emai::UniString fileName;
				for (EmaiUint32 i = 0; i < attachmentsCount; i++)
				{
					messagePartEnumerator.GetInlineAtIndex(i, fileName, contentId, NULL, length);
					attachmentInfo = [NSString stringWithFormat:
								@"%S (size = %d content ID = %S)", 
								fileName.c_str(), length, contentId.c_str()];
					data.mFormatedString = [attachmentInfo uniString];
					data.mFileName = fileName;
					data.mIsInline = true;
					data.mAttachmentIndex = i;
					mAttachments->push_back(data);
				}
			}
			[mAttachmentList reloadData];
			[self fillMessageTextView:messagePartEnumerator];
		}
		catch (Emai::Exception& exception)
		{
			[self showAlertForException:exception];
			[mAttachmentList reloadData];
			[mStatusText setStringValue:@"Error"];
			[mPerStatus setStringValue:@""];
		}
	}
}

- (IBAction)textFormatChanged:(id)sender
{
	if (mMessages->size() > 0)
	{
		Emai::Message message = mMessages->at([mMessageList selectedRow]);
		MessagePartEnumerator messagePartEnumerator(message);
		messagePartEnumerator.Enumerate();
		[self fillMessageTextView:messagePartEnumerator];
	}
}

- (int)numberOfRowsInTableView:(NSTableView *)aTable
{
	if (aTable == mMessageList)
		return mMessages->size();
	else
		return mAttachments->size();
}

- (id)tableView:(NSTableView *)aTable 
	 objectValueForTableColumn:(NSTableColumn *)aCol 
							   row:(int)aRow
{
	// Fill tables with data
	if (aTable == mMessageList)
	{
		Emai::Message message = mMessages->at(aRow);
		Emai::UniString subject;
		EmaiConstUniCharPtr sub = message.GetField(NULL, EmaiSubject);
		if (sub != NULL)
		{
			subject = sub;
			return [NSString stringWithCharacters:subject.c_str() length:subject.length()];
		}
		else return @"";
	}
	else
	{
		ItemData data = mAttachments->at(aRow);
		return [NSString stringWithCharacters:data.mFormatedString.c_str() 
						 length:data.mFormatedString.length()];
	}
}

- (IBAction)receiveMessages:(id)sender
{
	if (EmaiRegisterLibrary([[mLicKey stringValue] uniString]) == EmaiErrNoErr)
	{
		try	
		{
			Pop3Callback statusCallback;
			mMessages->clear();
			mAttachments->clear();
			[mFrom setStringValue:@""];
			[mTo setStringValue:@""];
			[mCC setStringValue:@""];
			[mMessage setString:@""];
			EmaiOptions options = EmaiNullOptions;
			if ([mUseSsl state]) 
				options = EmaiPop3UseSSLOption;
			Emai::Pop3Session pop3Session(statusCallback, 60, options);
			
			NSString* mailServer = [mMailServer stringValue];
			NSString* account = [mAccount stringValue];
			NSString* password = [mPassword stringValue];

			// Set handle of progress percent static text control as user data to change its in statusCallback
			EmaiUint32 percent = 0;
			pop3Session.SetUserData(Pop3PercentText, &mPerStatus, sizeof(NSTextField*));
			pop3Session.SetUserData(Pop3CurrentPercent, &percent, sizeof(EmaiUint32));
			pop3Session.SetUserData(Pop3StatusText, &mStatusText, sizeof(NSTextField*));
			pop3Session.SetUserData(Pop3CurrentMessage, &percent, sizeof(EmaiUint32));
			pop3Session.SetUserData(Pop3MessageNumber, &percent, sizeof(EmaiUint32));

			// Connect to pop3 server
			pop3Session.Connect([mailServer uniString], [mPort intValue], 
								EmaiNullOptions);

			// Authenticate on server
			EmaiUint32 authTypes[] = {
				EmaiPOP3AuthAutomatic,
				EmaiPOP3AuthLoginCommand,
				EmaiPOP3AuthApop,
				EmaiPOP3AuthLogin,
				EmaiPOP3AuthPlain,
				EmaiPOP3AuthCramMD5,
				EmaiPOP3AuthNTLM
			};
			
			pop3Session.Authenticate([account uniString], [password uniString], 
									 authTypes[[mAuthType indexOfSelectedItem]]);

			// Create history file path
			NSArray *paths = NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory,
								NSUserDomainMask, YES);
			NSString *destPath = [paths objectAtIndex:0];
			destPath = [destPath stringByAppendingString:@"/net.dacons.pop3demo.messagehistory"];
			
			Emai::MessageInfoList messageInfoList([destPath uniString]);

			pop3Session.RetrieveMessageList(messageInfoList);

			EmaiUint32 messageCount = messageInfoList.GetCount();
			pop3Session.SetUserData('COUN', &messageCount, sizeof(EmaiUint32));

			// Fill message list with messages
			EmaiUint32 currentMessageNumber = 0;

			while (currentMessageNumber < messageCount)
			{
				pop3Session.SetUserData('CURM', &currentMessageNumber, sizeof(EmaiUint32));
				EmaiMessageInfo outInfo;
				messageInfoList.GetInfo(currentMessageNumber, outInfo);
			
				Emai::Message message = pop3Session.RetrieveMessage
							(outInfo.messageNumber);
				mMessages->push_back(message);
				
				// Mark message received
				messageInfoList.MarkMessageReceived(++currentMessageNumber);
			}
			
			// Remove message from server if chose
			if ([mRemoveMessages state] == 1)
			{
				EmaiUint32 numDays[] = { 1, 7, 31 };

				messageInfoList.EvaluateSublist(numDays[[mTimePeriods indexOfSelectedItem]],
												EmaiMessagesForDeletion);

				currentMessageNumber = 0;
				messageCount = messageInfoList.GetCount(EmaiMessagesForDeletion);
				pop3Session.SetUserData('COUN', &messageCount, sizeof(EmaiUint32));

				while (currentMessageNumber < messageCount)
				{
					pop3Session.SetUserData('CURM', &currentMessageNumber, sizeof(EmaiUint32));
					EmaiMessageInfo outInfo;
					messageInfoList.GetInfo(currentMessageNumber, outInfo, 
											EmaiMessagesForDeletion);

					// Delete message with number outInfo.messageNumber
					pop3Session.DeleteMessage(outInfo.messageNumber);
					currentMessageNumber++;
				}
			}
			[mRemoveMessages setState:0];
			[mTimePeriods setEnabled:0];
			[mPerStatus setStringValue:@""];
			
			// Close session and disconnect from server
			pop3Session.Disconnect();
		}
		catch (Emai::Exception& exception)
		{
			[mMessageList reloadData];
			[mAttachmentList reloadData];
			[self showAlertForException:exception];
			[mPerStatus setStringValue:@""];
			[mStatusText setStringValue:@""];
			[mPerStatus displayIfNeeded];
			[mStatusText displayIfNeeded];
		}
		[mMessageList reloadData];
		[mAttachmentList reloadData];
	}
	else
		[self showAlertWithMessageText:@"Licence key is invalid"
						   description:@"Restart application and enter valid key"];
}

- (IBAction)setRemoveAction:(id)sender
{
	[mTimePeriods setEnabled:[mRemoveMessages state]];
}

- (IBAction)saveAttachment:(id)sender
{
	if (mMessages->size() > 0)
	{
		if ([mAttachmentList selectedRow] >= 0)
		{
			Emai::Message message = mMessages->at([mMessageList selectedRow]);
			MessagePartEnumerator messagePartEnumerator(message);
			messagePartEnumerator.Enumerate();
			if ((messagePartEnumerator.GetAttachmentsCount() > 0) || (messagePartEnumerator.GetInlinesCount() > 0))
			{
				ItemData data = mAttachments->at([mAttachmentList selectedRow]);
				Emai::UniString fileName;
				Emai::UniString filePath;
				[self getAttachment:[NSString stringWithCharacters:data.mFileName.c_str() length:data.mFileName.length()]
									  path:filePath fileName:fileName];
				messagePartEnumerator.SaveAttachmentAtIndex(data.mIsInline, data.mAttachmentIndex, filePath.c_str(), fileName.c_str());
			}
		}
	}
}
@end
