/*
 // @file		SmtpWindowController.mm
 //
 // @version		Mailit for C/C++ 1.0 sample file
 //
 // @author		(C) 2007 Dacons LLP.
 //
 // @bug			If you have problems or suggestions, please contact
 //				the Online Support at: http://www.dacons.net/support
 */


#import "SmtpWindowController.h"

#include PCH_FILE_NAME

#import "stringext.h"
#include "SmtpCallback.h"
#include <EmaiSmtp.hpp>
#include <EmaiMessage.hpp>
#include <EmaiMessage.h>
#include <MessageComposer.hpp>
#include "ErrorsDescriptions.hpp"


@implementation SmtpWindowController

- (id)init
{
	self = [super initWithWindowNibName:@"Window"];
	if (self == NULL)
		return self;
	mAttachmentPathsList = [[NSMutableArray alloc] init];
	mAttachmentTypesList = [[NSMutableArray alloc] init];
	[self window];
	[mAttachments setDoubleAction:@selector(attachmentDoubleClick:)];		
	return self;
}

- (void)dealloc
{
	[mAttachmentPathsList release];
	[mAttachmentTypesList release];
	[super dealloc];
}


-(void) windowDidLoad
{
	[mMailServer setStringValue:@"mail.server.net"];
	[mAccount setStringValue:@"your.account"];
	[mPassword setStringValue:@"password"];
	[mSubject setStringValue:@"Subject"];
	[mTo setStringValue:@"recipient@othermail.server.net"];
	[mFrom setStringValue:@"sender@mail.server.net"];
	[mHtmlText setString:@"<html><body>Message text</body></html>"];
	[mPlainText setString:@"Message text"];
	[mPort setIntValue:EmaiDefaultSmtpPort];
}

-(void) showAlertForException:(Emai::Exception&) inException
{	
	NSAlert *alert = [NSAlert alertWithMessageText:@"An error occured while communicating with smtp server"
									 defaultButton:@"OK" alternateButton:NULL otherButton:NULL
						 informativeTextWithFormat:@"%@", errorsDescriptions[::abs(inException.GetErrorCode())]];
	[alert setAlertStyle:NSCriticalAlertStyle];
	[alert runModal];
}

-(void) showAlertWithMessageText:(NSString*) inInfText description:(NSString*) inMsgText
{
	NSAlert *alert = [NSAlert alertWithMessageText:inInfText
									 defaultButton:@"OK" alternateButton:NULL otherButton:NULL
						 informativeTextWithFormat:@"%@", inMsgText];
	[alert setAlertStyle:NSWarningAlertStyle];
	[alert runModal];
}

- (IBAction)addAttachment:(id)sender
{
	// Add attachments to attachments list using open panel
	int result;
    NSOpenPanel* openPanel = [NSOpenPanel openPanel];
 
    [openPanel setAllowsMultipleSelection:YES];
    [openPanel setTitle:@"Choose Attachmet"];
    [openPanel setMessage:@"Choose files to add to message."];
    [openPanel setDelegate:self];
    result = [openPanel runModalForDirectory:NSHomeDirectory() file:nil types:nil];
    if (result == NSOKButton) 
	{
        NSArray* filesToAdd = [openPanel filenames];
        int i, count = [filesToAdd count];

        for (i = 0; i < count; i++) 
		{
			[mAttachmentPathsList addObject:[filesToAdd objectAtIndex:i]];
			[mAttachmentTypesList addObject:[NSNumber numberWithInt: 0]];
        }
		[mAttachments reloadData];
    }

}

- (IBAction)delAttachment:(id)sender
{
	// Delete attachment from list
	if ([mAttachmentPathsList count] > 0)
	{	
		[mAttachmentPathsList removeObjectAtIndex:[mAttachments selectedRow]];
		[mAttachmentTypesList removeObjectAtIndex:[mAttachments selectedRow]];
		[mAttachments reloadData];
	}
}

- (IBAction)sendMessage:(id)sender
{
	if (EmaiRegisterLibrary([[mLicKey stringValue] uniString]) == EmaiErrNoErr)
	{
		try
		{
			[mStatusText setStringValue:@"Composing message..."];
			[mStatusText displayIfNeeded];
			/*Create the message from different parts*/
			MessageComposer messageComposer;
			
			if ([mAttachText state])
				messageComposer.SetUnicodeText([[mPlainText string] uniString]);

			// Add attachments to the message
			NSFileManager *fileManager = [NSFileManager defaultManager];
			NSString *filePath = NULL;
			NSString *fileName = NULL;
			NSString* left = NULL;
			NSString* right = NULL;
			NSString* tmp = NULL;
			NSRange range = [[mHtmlText string] rangeOfString:@"</body>"];
			left = [[mHtmlText string] substringToIndex:range.location];
			right = [[mHtmlText string] substringFromIndex:range.location];
			NSMutableString *htmlText = [NSMutableString string];
			[htmlText appendString:left];
			EmaiUint32 attachmentCount = [mAttachmentPathsList count];
			if (attachmentCount > 0)
			{
				for (int i = 0; i <= attachmentCount - 1; i++)
				{
					filePath = [mAttachmentPathsList objectAtIndex:i];
					fileName = [fileManager displayNameAtPath:filePath];
					if ([[mAttachmentTypesList objectAtIndex:i] compare: [NSNumber numberWithInt: 1]] 
						== NSOrderedSame)
					{
						// Formatting html tag for inline attachment
						tmp = [NSString stringWithFormat:@"<img src=\"cid:%@\">", 
							fileName];
						NSRange tmpRange = [htmlText rangeOfString:tmp];
						if (tmpRange.length == 0)
							[htmlText appendString:tmp];
						
						messageComposer.AddInlineFromFile([fileName uniString], 
									[filePath uniString], [fileName uniString]);
					}
					else
						messageComposer.AddAttachmentFromFile([fileName uniString], 
									[filePath uniString]);
				}
			}
			[htmlText appendString:right];
			if ([mAttachHtml state])
				messageComposer.SetHtml([htmlText UTF8String]);

			/*Add contact fields to the message*/
			Emai::ContactList contactTo;
			contactTo.AddContact([[mTo stringValue] uniString], NULL);
			Emai::ContactList contactFrom;
			contactFrom.AddContact([[mFrom stringValue] uniString], NULL);

			Emai::Message message = messageComposer.ComposeMessage(contactFrom, 
				NULL, contactTo,
				NULL, NULL, [[mSubject stringValue] uniString]);

			/*Check if message is created successufully*/
			if (message != NULL)
			{
				/*Create the session*/
				SmtpCallback statusCallback;
				EmaiOptions options = EmaiNullOptions;
				if ([mUseSsl state]) 
					options = EmaiSmtpUseSSLOption;
				Emai::SmtpSession smtpSession(statusCallback, 60, options);
				
				int perc = 0;
				smtpSession.SetUserData(SmtpCurrentPercent, &perc, sizeof(EmaiUint32));
				smtpSession.SetUserData(SmtpStatusText, &mStatusText, sizeof(NSTextField*));
				// Set handle of progress percent control as user data to change its in statusCallback
				smtpSession.SetUserData(SmtpPercentText, &mPerStatus, sizeof(NSTextField*));
				
				/*Connect to the server*/
				smtpSession.Connect([[mMailServer stringValue] uniString], [mPort intValue]);
				
				/*Authenticate the session*/
				EmaiUint32 authTypes[] = {
					EmaiSMTPAuthAutomatic,
					EmaiSMTPAuthLogin,
					EmaiSMTPAuthPlain,
					EmaiSMTPAuthCramMD5,
					EmaiSMTPAuthNTLM
				};
				
				smtpSession.Authenticate([[mAccount stringValue] uniString], 
										 [[mPassword stringValue] uniString], 
										 authTypes[[mAuthType indexOfSelectedItem]]);
				/*Send the message*/
				smtpSession.SendMessage(message);
				
				/*Disconnect from the server*/
				smtpSession.Disconnect();
			}
			[mStatusText setStringValue:@""];
			[mPerStatus setStringValue:@""];
			[mPerStatus displayIfNeeded];
			[mStatusText displayIfNeeded];
		}
		catch (Emai::Exception& exception)
		{
			[self showAlertForException:exception];
			[mPerStatus setStringValue:@""];
			[mStatusText setStringValue:@""];
			[mPerStatus displayIfNeeded];
			[mStatusText displayIfNeeded];
		}
	}
	else
		[self showAlertWithMessageText:@"Licence key is invalid"
					description:@"Restart application and enter valid key"];
}

- (int)numberOfRowsInTableView:(NSTableView *)aTable
{
	return [mAttachmentPathsList count];
}

- (void)tableView:(NSTableView *)tableView setObjectValue:(id)object forTableColumn:(NSTableColumn *)aCol row:(int)row;
{
	if ([[aCol identifier] isEqualToString:@"INLINE"])
		[mAttachmentTypesList replaceObjectAtIndex:row withObject: object];
}

- (id)tableView:(NSTableView *)aTable 
	 objectValueForTableColumn:(NSTableColumn *)aCol 
							   row:(int)aRow

{
	// Fill tables with data
	if ([[aCol identifier] isEqualToString:@"INLINE"])
		return [mAttachmentTypesList objectAtIndex:aRow];
	else
		return [mAttachmentPathsList objectAtIndex:aRow];
}

- (IBAction)attachmentDoubleClick:(id)sender
{
	if ([mAttachHtml state])
	{
		EmaiUint32 index = [mAttachments selectedRow];
		NSString *filePath = [mAttachmentPathsList objectAtIndex:index];
		[mHtmlText insertText:[NSString stringWithFormat:@"<img src=\"cid:%@\">", 
								[filePath lastPathComponent]]];
		[mAttachmentTypesList replaceObjectAtIndex:index withObject:[NSNumber numberWithInt: 1]]; 
		[mAttachments reloadData];
	}
	else
		[self showAlertWithMessageText:@"An error occured while adding inline file"
					description:@"You can not add inline attachment without html message part"];
}

- (IBAction)attachText:(id)sender
{
	[mPlainText setEditable:[mAttachText state]];
}

- (IBAction)attachHtml:(id)sender
{
	[mHtmlText setEditable:[mAttachHtml state]];
}
@end
