/*!
// @file		EmaiExceptions.hpp
//
// @brief		Exceptions class
//
// @version		Mailit for C/C++ 1.0
//
// @author		(C) 2008 Dacons LLP.
//
// @bug			If you have problems or suggestions, please contact
//				the Online Support at: http://www.mailitcomponents.com/support
*/

#ifndef __EMAIEXCEPTIONS_HPP__
#define __EMAIEXCEPTIONS_HPP__

#include <EmaiErrors.h>

/*!
// @namespace	Emai
// @brief		Mailit for C/C++ library namespace
// @author		(C) 2006 Dacons LLP.
*/namespace Emai
{

/*!
// @class	Exception
// @brief	General exceptions class.
*/
class Exception
{
protected:

	EmaiError				mErrorCode;

										Exception();
public:
	
										~Exception()
										{
										}
										Exception(EmaiError inErrorCode):
										mErrorCode(inErrorCode)
										{
										}

	/*!
	// @brief	Get error code
	//
	// @return	Result code (See @ref EmaiEnumResultCodes)
	*/
	EmaiError							GetErrorCode() const
										{
											return mErrorCode;
										}
};

/*!
// @class	CommandFailedException
// @brief	This class is used, when the server returns an error response.
//			I. e. the command sent to the server is not supported by server or
//			parameters specified for the command are invalid.
*/
class CommandFailedException : public Exception
{
protected:

public:
	
										~CommandFailedException()
										{
										}
										CommandFailedException():
										Exception(EmaiErrCommandFailed)
										{
										}
};

/*!
// @class	CallNotInContextException
// @brief	This class is used, when the command sent to server is not allowed
//			within current context. For instance, you are trying to fetch
//			message data while no mailbox is selected on server.
*/
class CallNotInContextException : public Exception
{
protected:

public:
	
										~CallNotInContextException()
										{
										}
										CallNotInContextException():
										Exception(EmaiErrCallNotInContext)
										{
										}
};

/*!
// @class	CommandCancelledException
// @brief	This class is used, when the user has cancelled the action.
*/
class CommandCancelledException : public Exception
{
protected:

public:
	
										~CommandCancelledException()
										{
										}
										CommandCancelledException():
										Exception(EmaiErrInterruptedByUser)
										{
										}
};

inline void
CheckErrorCode(
		EmaiError inErrorCode)
{
	switch (inErrorCode)
	{
		case EmaiErrNoErr:
			break;
		case EmaiErrCommandFailed:
			throw CommandFailedException();
			break;
		case EmaiErrCallNotInContext:
			throw CallNotInContextException();
			break;
		case EmaiErrInterruptedByUser:
			throw CommandCancelledException();
			break;
		default:
			throw Exception(inErrorCode);
			break;
	}
}

}
#endif /*__EMAIEXCEPTIONS_HPP__*/
